#include <Servo.h>                // Biblioteca pentru servo-motor

#define SERVO 9
#define ECHO_PIN_DIST 8
#define TRIGGER_PIN_DIST 7
#define ECHO_PIN_FULL  4
#define TRIGGER_PIN_FULL  5
#define BUZZER 6
#define LED_ROSU 13
#define LED_GALBEN 12
#define LED_VERDE 11
#define FULL 5
#define MEDIUM 15
#define BUTTON 2
#define MIN_DIST 5
#define MAX_DIST 20

Servo my_servo;

// Starile butonului
int last_state = HIGH;
int current_state;

void setup()
{
  my_servo.attach(SERVO);
  pinMode(TRIGGER_PIN_DIST, OUTPUT);
  pinMode(ECHO_PIN_DIST, INPUT);
  pinMode(TRIGGER_PIN_FULL, OUTPUT);
  pinMode(ECHO_PIN_FULL, INPUT);
  pinMode(LED_ROSU, OUTPUT);
  pinMode(LED_GALBEN, OUTPUT);
  pinMode(LED_VERDE, OUTPUT);
  pinMode(BUTTON, INPUT_PULLUP);
  delay(100);
  Serial.begin(9600);
}

// Resetez trigger-ul in starea de low
void reset_trigger(int trigger_pin) 
{
  digitalWrite(trigger_pin, LOW);
  delayMicroseconds(5000);
  digitalWrite(trigger_pin, HIGH);
  delayMicroseconds(5000);
  digitalWrite(trigger_pin, LOW);
}

// Calculeaza distanta pana la un obiect
float get_distance(int echo_pin) 
{
  float dist;
  
  // Setez pinul
  pinMode(echo_pin, INPUT);

  // Masor distanta pana la mana
  long duration = pulseIn(echo_pin, HIGH);
  // Distanta in cm (convertire timp - distanta)
  dist = duration / 58.82;
  return dist;
}

// Deschide/ inchide capacul in functie
// de distanta la care se afla obiectul
void open_trash_by_hand(float dist) 
{
  int pos;

  if (dist <= MAX_DIST && dist >= MIN_DIST) {
    pos = 90;
    my_servo.write(pos);
    delay(1000);
  }
  else {
    pos = 0;
    my_servo.write(pos);
    delay(50);
    if (dist < 2) {
      tone(BUZZER, 1000, 500);
    }
  }
}

void open_trash_by_button() {
   int pos;

   // Citesc starea curenta a butonului
   current_state = digitalRead(BUTTON);

   if (last_state == LOW && current_state == HIGH) {
    pos = 90;
    my_servo.write(pos);
    delay(5000);
    pos = 0;
    my_servo.write(pos);
    delay(50);
  }

  // Salvez ultima stare
  last_state = current_state;
}

// Verifica cat de plin e cosul
void check_content(float dist_full) {
   if (dist_full <= FULL) {
    // Aprinde ledul corespunzator
    digitalWrite(LED_ROSU, HIGH);
    delay(100);
    digitalWrite(LED_GALBEN, LOW);
    delay(100);
    digitalWrite(LED_VERDE, LOW);
    delay(100);
    tone(BUZZER, 1000, 500);
    delay(1000);
  }
  else {
    digitalWrite(LED_ROSU, LOW);
    if (dist_full > FULL && dist_full < MEDIUM) {
       digitalWrite(LED_GALBEN, HIGH);
       digitalWrite(LED_VERDE, LOW);
       delay(1000);
    }
    else {
       digitalWrite(LED_GALBEN, LOW);
       digitalWrite(LED_VERDE, HIGH);
       delay(1000);
    }
  }
}

void loop()
{
  // Masor distanta pana la mana
  reset_trigger(TRIGGER_PIN_DIST);
  float dist = get_distance(ECHO_PIN_DIST);

  // Masor distanta in interiorul cosului
  // pana la gunoi
  reset_trigger(TRIGGER_PIN_FULL);
  float dist_full = get_distance(ECHO_PIN_FULL);

  // Printez distanta pana la gunoi pe seriala
  Serial.println(dist_full);

  // Daca s-a actionat manual deschiderea cosului
  open_trash_by_button();

  //  Deschidere cos declansata de mana aflata in fata capacului
  open_trash_by_hand(dist);

  // Verificare umplere cos
  check_content(dist_full);
}
